import QtQuick 2.0
import Ubuntu.Components 1.1

FocusScope {
    id: root

    property var windowData

    /* Size/position */
    x: windowData.x
    y: windowData.y
    width: windowData.windowWidth
    height: windowData.windowHeight
    property int minWidth: windowData.minWidth
    property int maxWidth: windowData.maxWidth
    property int minHeight: windowData.minHeight
    property int maxHeight: windowData.maxHeight
    property bool resizable: windowData.resizable
    property bool movable: windowData.movable // false implies is anchored to parent

    /* Decoration */
    property string decoration: windowData.decoration // normal, small, none, client-specified
    property bool active: windowData.active
    property string title: windowData.title
    property bool canBeMaximized: windowData.canBeMaximized
    property bool canBeMinimized: windowData.canBeMinimized
    property bool canBeClosed: windowData.canBeClosed

    /* Other info */
    property string type: windowData.type     // regular, floating, dialog, satellite, popup, gloss, tip, freestyle (for info purposes only)
    property string windowState: windowData.state   // normal, maximized, minimized, fullscreen
    property bool interactive: windowData.interactive // may not be necessary, could go in MirSurface instead
    property int parentId: windowData.parentId

    signal windowWidthChanged(int windowWidth)
    signal windowHeightChanged(int windowHeight)
    signal focusRequested

    focus: interactive

    BorderImage {
        anchors {
            fill: root
            margins: -units.gu(2)
        }
        source: "dropshadow2gu.sci"
        opacity: .3
        Behavior on opacity { UbuntuNumberAnimation {} }
    }

    WindowDecoration {
        id: decoration
        anchors { left: parent.left; top: parent.top; right: parent.right }
        height: units.gu(3)
        title: root.title + " " + root.type
        onClose: root.close();
        onMaximize: root.maximize();
        onMinimize: root.minimize();
    }

    Rectangle {
        id: content
        anchors { left: parent.left; top: decoration.bottom; right: parent.right; bottom: parent.bottom }
        color: root.activeFocus ? "lightblue" : "lightgrey"
    }


    MouseArea {
        anchors.fill: parent
        property int resizeMargin: 30
        anchors.margins: -resizeMargin

        property bool resizeWidth: false
        property bool resizeHeight: false

        property int startX: 0
        property int startWidth: 0
        property int startY: 0
        property int startHeight: 0

        onPressed: {
            if (interactive) {
                focusRequested();
            }
            if (root.resizable) {
                if (mouseX > width - 2*resizeMargin) {
                    resizeWidth = true;
                    startX = mouseX;
                    startWidth = root.width;
                }
                if (mouseY > height - 2*resizeMargin) {
                    resizeHeight = true;
                    startY = mouseY;
                    startHeight = root.height;
                }
            }
            if (!resizeHeight && !resizeWidth && root.movable) {
                drag.target = root; // FIXME: breaks the model x,y binding
            }
        }

        onMouseXChanged: { //TODO: this should resize a ghost surface/outline, while calling a requestResize on the
            // underlying MirSurface, which will update the windowModel when resized surface ready to be rendered.
            if (resizeWidth) {
                var width = startWidth + (mouseX - startX);
                if (width <= root.maxWidth && width >= root.minWidth) {
                    root.windowWidthChanged(startWidth + (mouseX - startX))
                }
            }
        }
        onMouseYChanged: {
            if (resizeHeight) {
                var height = startHeight + (mouseY - startY);
                if (height <= root.maxHeight && height >= root.minHeight) {
                    root.windowHeightChanged(startHeight + (mouseY - startY))
                }
            }
        }

        onReleased: {
            resizeWidth = false;
            resizeHeight = false;
            drag.target = undefined;
        }
    }

}
